<?php
/*
Plugin Name: TPC! Memory Usage
Description: Used to keep track of WordPress memory usage.
Version: 0.1
Author: Chris Strosser
Author URI: http://webjawns.com
*/

// Prevent direct calls to this file
if ( !function_exists('add_action') ) {
	header('Status: 403 Forbidden');
	header('HTTP/1.1 403 Forbidden');
	exit();
}

// Create default key and status
add_option('tpc_memory_usage_location', 'footer');
add_option('tpc_memory_usage_graph', 'peak');
add_option('tpc_memory_usage_admin_footer', true);

class tpcmem {
	/**
	 * TPC! Memory Usage plugin version
	 * 
	 * @access private
	 * @var string
	 */
	var $version = '0.1';
	
	/**
	 * Last recorded WordPress memory usage
	 * 
	 * @var int
	 */
	var $memory_usage;
	
	/**
	 * Last recorded WordPress peak memory usage
	 * 
	 * @var int
	 */
	var $memory_peak_usage;
	
	/**
	 * PHP Memory Limit
	 * 
	 * @var string|int
	 */
	var $php_memory_limit;
	
	/**
	 * PHP5 constructor
	 * 
	 * @return void
	 */
	function __construct() {
		$this->tpcmem();
	}
	
	/**
	 * PHP4 style constructor
	 * 
	 * @return void
	 */
	function tpcmem() {
		// Initialize variables when class is initiated...
		$this->get_usage(true);
		$this->get_peak_usage(true);
		
		$this->php_memory_limit = ( @ini_get('memory_limit') ) ? ini_get('memory_limit') : 'N/A';
		
		// Add hook for displaying memory usage in HTML
		$this->display_usage_hook();
	}
	
	/**
	 * Get current memory usage for script.
	 * 
	 * @param bool $set_var When true, sets the $tpcmem->memory_usage variable. Otherwise returns current value of $tpcmem->memory_usage.
	 * @return int Current memory usage.
	 */
	function get_usage($set_var = false) {
		if ( function_exists('memory_get_usage') && $set_var ) {
			$memory_usage = memory_get_usage();
			$this->memory_usage = $memory_usage;
		} else {
			$this->memory_usage = 0;
		}
		
		return $this->memory_usage;
	}
	
	/**
	 * Get peak memory usage for script.
	 * 
	 * @param bool $set_var When true, sets the $tpcmem->memory_peak_usage variable. Otherwise returns current value of $tpcmem->memory_peak_usage.
	 * @param $real_usage
	 * @return int Peak memory usage.
	 */
	function get_peak_usage($set_var = false, $real_usage = false) {
		if ( function_exists('memory_get_peak_usage') && $set_var ) {
			if ( $real_usage )
				$memory_peak_usage = memory_get_peak_usage(true);
			else
				$memory_peak_usage = memory_get_peak_usage();
			
			$this->memory_peak_usage = $memory_peak_usage;
		} else {
			$this->memory_peak_usage = 0;
		}
		
		return $this->memory_peak_usage;
	}
	
	/**
	 * Get the current memory limit for PHP.
	 * 
	 * @param $wp_limit When set to true, returns WP_MEMORY_LIMIT. When false, returns memory_limit in php.ini.
	 * @return string|int The memory limit. (May be in either format: 32M or 32, 64M or 64, etc.)
	 */
	function get_limit($wp_limit = true) {
		if ( $wp_limit )
			return WP_MEMORY_LIMIT;
		else
			return $this->php_memory_limit;
	}
	
	/**
	 * Returns the default value for the memory meter.
	 * 
	 * Defaults to current usage if graph is turned off.
	 * 
	 * @return int Memory usage (peak or current depending on option value).
	 */
	function get_graph_default() {		
		switch ( get_option('tpc_memory_usage_graph') ) {
			case 'peak':
				$mem = $this->get_peak_usage(true);
				break;
			
			case 'current':
			default:
				$mem = $this->get_usage(true);
				break;
		}
		
		return $mem;
	}
	
	//
	// Actions
	//
	
	/**
	 * Add hook for displaying memory usage comment in HTML.
	 * 
	 * @uses get_option()
	 * @uses add_action()
	 */
	function display_usage_hook() {
		switch ( get_option('tpc_memory_usage_location') ) {
			case 'header':
				add_action('wp_header', '_tpc_memory_usage');
				add_action('admin_head', '_tpc_memory_usage');
				break;
			
			case 'footer':
				add_action('wp_footer', '_tpc_memory_usage');
				add_action('admin_footer', '_tpc_memory_usage');
				break;
			
			default:
				// Don't display anywhere in comments
				break;
		}
	}
	
	//
	// Template Functions
	//
	
	function get_display_locations() {
		$locations = array('header' => 'Header', 'footer' => 'Footer', 'none' => 'Do not display');
		return $locations;
	}
	
	function get_graph_options() {
		$options = array('current' => 'Current memory usage', 'peak' => 'Peak memory usage', 'hide' => 'Do not display');
		return $options;
	}
	
	function get_admin_footer_options() {
		$options = array(0 => 'No', 1 => 'Yes');
		return $options;
	}
	
	/**
	 * Displays current value of the memory usage variable.
	 * 
	 * @return void
	 */
	function display_usage() {
		echo $this->memory_usage;
	}
	
	/**
	 * Displays current value of the peak memory usage variable.
	 * 
	 * @return void
	 */
	function display_peak_usage() {
		echo $this->memory_peak_usage;
	}
	
	function dropdown_display_locations($selected = false) {
		$d = ''; // default
		$l = ''; // other location(s)
		
		$locations = $this->get_display_locations();
		
		foreach ( (array)$locations as $key => $loc ) {
			$key = esc_attr($key);
			$loc = esc_html($loc);
			
			if ( $key == $selected )
				$d = "<option value='$key' selected='selected'>$loc</option>";
			else
				$l .= "<option value='$key'>$loc</option>";
		}
		
		echo $d . $l;
	}
	
	function dropdown_graph_options($selected = false) {
		$d = ''; // default
		$o = ''; // other options
		
		$options = $this->get_graph_options();
		
		foreach ( (array)$options as $key => $opt ) {
			$key = esc_attr($key);
			$opt = esc_html($opt);
			
			if ( $key == $selected )
				$d = "<option value='$key' selected='selected'>$opt</option>";
			else
				$o .= "<option value='$key'>$opt</option>";
		}
		
		echo $d . $o;
	}
	
	function dropdown_admin_footer($selected = false) {
		$d = ''; // default
		$o = ''; // other options
		
		$options = $this->get_admin_footer_options();
		
		foreach ( (array)$options as $key => $opt ) {
			$key = esc_attr($key);
			$opt = esc_html($opt);
			
			if ( $key == $selected )
				$d = "<option value='$key' selected='selected'>$opt</option>";
			else
				$o .= "<option value='$key'>$opt</option>";
		}
		
		echo $d . $o;
	}
	
}

/**
 * Initializes the TPC! Memory Usage plugin.
 * 
 * @uses add_action('admin_menu', 'tpc_memory_usage_config_page');
 */
function tpc_memory_usage_init() {
	global $tpcmem;
	
	$tpcmem = new tpcmem();

	add_action('admin_menu', 'tpc_memory_usage_config_page');
	add_action('admin_init', 'tpc_memory_usage_admin_init');
}
add_action('init', 'tpc_memory_usage_init');

/**
 * Initializes the hooks and options for the administration panel.
 * 
 * @return void
 */
function tpc_memory_usage_admin_init() {
	global $tpcmem;
	
	if ( !function_exists('plugins_url') ) return;

	// Register scripts and styles
	wp_register_script('tpc-memory-usage', plugins_url('tpc_memory_usage/tpc_memory_usage.js'), array('jquery'), $tpcmem->version, true);
	wp_register_style('tpc-memory-usage', plugins_url('tpc_memory_usage/tpc_memory_usage.css'), array(), $tpcmem->version);
	
	// Register hooks and filters
	add_action('wp_dashboard_setup', 'tpc_memory_usage_dashboard');
	add_filter('admin_footer_text', 'tpc_memory_usage_admin_footer');
	add_filter('tpc_megabytes', 'tpc_bytes_to_mb');
	add_filter('tpc_megabytes', create_function('$a', '{ return $a . "M"; }'));
	
	if ( function_exists('register_setting') ) {
		register_setting('tpc_memory_usage', 'tpc_memory_usage_location');
		register_setting('tpc_memory_usage', 'tpc_memory_usage_graph');
		register_setting('tpc_memory_usage', 'tpc_memory_usage_admin_footer');
	}
}

function tpc_memory_usage_admin_footer($html) {
	global $tpcmem;
	
	if ( get_option('tpc_memory_usage_admin_footer') ) {
		$memory = $tpcmem->get_graph_default();
		$html .= ' | Memory Usage: ' . apply_filters( 'tpc_megabytes', $memory ) . '/' . $tpcmem->get_limit();
	}
	
	return $html;
}

/**
 * Adds TPC! Memory Usage configuration page to plugins.php admin menu.
 * 
 * @uses add_options_page()
 */
function tpc_memory_usage_config_page() {
	$plugin_page = add_options_page(__('TPC! Memory Usage'), 'Memory Usage', 8, basename(__FILE__), 'tpc_memory_usage_conf');
}

function tpc_memory_usage_conf() {
	global $tpcmem;
	// If we completed post, store updated options
	if ( isset($_POST['tpc_memory_usage_update']) ) {
		// Update the HTML comment location
		$tpcmem_location = $_POST['tpc_memory_usage_location'];
		update_option('tpc_memory_usage_location', $tpcmem_location);
		
		// Update the dashboard graph value to use
		$tpcmem_graph_value = $_POST['tpc_memory_usage_graph'];
		update_option('tpc_memory_usage_graph', $tpcmem_graph_value);
		
		// Update the "display in admin footer" setting
		$tpcmem_admin_footer = (int)$_POST['tpc_memory_usage_admin_footer'];
		update_option('tpc_memory_usage_admin_footer', $tpcmem_admin_footer);
	}
?>
<div class="wrap">
<?php if ( function_exists('screen_icon') ) screen_icon(); ?>
<h2><?php _e('TPC! Memory Usage Settings'); ?></h2>

<form method="post" action="options-general.php?page=tpc_memory_usage.php">

<h3><?php _e('Basic Settings'); ?></h3>
<table class="form-table" cellspacing="2" cellpadding="5" width="99%">
	<tr>
		<th scope="col" width="30%" valign="top"><label for="tpc_memory_usage_location"><?php esc_html_e('Show memory usage comment in'); ?>:</label></th>
		<td>
			<select name="tpc_memory_usage_location" id="tpc_memory_usage_location">
			<?php $tpcmem->dropdown_display_locations( get_option('tpc_memory_usage_location') ); ?>
			</select>
		</td>
	</tr>
	<tr>
		<th valign="top"><label for="tpc_memory_usage_graph"><?php esc_html_e('Value to use for dashboard graph'); ?>:</label></th>
		<td>
			<select name="tpc_memory_usage_graph" id="tpc_memory_usage_graph">
			<?php $tpcmem->dropdown_graph_options( get_option('tpc_memory_usage_graph') ); ?>
			</select>
		</td>
	</tr>
	<tr>
		<th valign="top"><label for="tpc_memory_usage_admin_footer"><?php esc_html_e('Show memory usage in admin footer'); ?></label></th>
		<td>
			<select name="tpc_memory_usage_admin_footer" id="tpc_memory_usage_admin_footer">
			<?php $tpcmem->dropdown_admin_footer( get_option('tpc_memory_usage_admin_footer') ); ?>
			</select>
		</td>
	</tr>
</table>

<p class="submit">
	<?php if ( function_exists('settings_fields') ) settings_fields('tpc_memory_usage'); ?>
	<input type="submit" name="tpc_memory_usage_update" value="<?php echo esc_attr('Save Changes'); ?>" />
</p>

</form>

</div>
<?php
}

/**
 * Add TPC! Memory Usage dashboard widget.
 * 
 * @uses wp_add_dashboard_widget()
 */
function tpc_memory_usage_dashboard() {
	wp_enqueue_style('tpc-memory-usage');
	wp_add_dashboard_widget('tpc_memory_usage_dashboard', 'Memory Usage Overview', '_tpc_memory_usage_dashboard');
}

/**
 * Display TPC! Memory Usage dashboard HTML.
 * 
 * @return void
 */
function _tpc_memory_usage_dashboard() {
	global $tpcmem;
	
	add_action('admin_footer', '_tpc_memory_usage_dashboard_js');
	
	$memory_usage = round( $tpcmem->get_usage(true) / 1024 / 1024, 2 );
	$memory_peak_usage = round( $tpcmem->get_peak_usage(true) / 1024 / 1024, 2 );
	
	$memory_usage_percent = round( $memory_usage / $tpcmem->get_limit() * 100, 0 );
	$memory_peak_usage_percent = round( $memory_peak_usage / $tpcmem->get_limit() * 100, 0 );
	
	$memory_usage_color = '#1C6280';
	if ( $memory_usage_percent > 70 )
		$memory_usage_color = '#CCCC00';
	if ( $memory_usage_percent > 90 )
		$memory_usage_color = '#CC0000';
	
	if ( !empty($memory_usage_percent) ) {
		$memory_usage_percent_string = ' <span style="color:' . $memory_usage_color . ';">(' . $memory_usage_percent . '%)</span>';
		$memory_peak_usage_percent_string = ' <span style="color:' . $memory_usage_color . ';">(' . $memory_peak_usage_percent . '%)</span>';
	}
?>

<ul>
	<li><strong><?php esc_html_e('Memory Usage'); ?></strong>: <span><?php echo esc_html( $memory_usage . 'MB' ); ?></span><?php echo $memory_usage_percent_string; ?></li>
	<li><strong><?php esc_html_e('Peak Memory Usage'); ?></strong>: <span><?php echo esc_html( $memory_peak_usage . 'MB' ); ?></span><?php echo $memory_peak_usage_percent_string; ?></li>
	<li><strong><?php esc_html_e('WP Memory Limit'); ?></strong>: <span><?php echo $tpcmem->get_limit(); ?></span></li>
	<li><strong><?php esc_html_e('PHP Memory Limit'); ?></strong>: <span><?php echo $tpcmem->get_limit(false); ?></span></li>
</ul>
<?php
	
	$memory_graph_value = get_option('tpc_memory_usage_graph');
	if ( $memory_graph_value == 'current' )
		$_graph_percent = $memory_usage_percent;
	elseif ( $memory_graph_value == 'peak' )
		$_graph_percent = $memory_peak_usage_percent;
	else
		$_graph_percent = false;

if ( false !== $_graph_percent ): ?>
<div class="progressbar">
<div class="widget" id="tpcmem_pbar_wrap">
<div class="widget" id="tpcmem_pbar" style="width: <?php echo $_graph_percent; ?>%; background-color: <?php echo $memory_usage_color; ?>;">
	<div id="tpcmem_pbar_percent"><?php echo $_graph_percent; ?>%</div>
</div><!-- #tpcmem_pbar -->
</div><!-- #tpcmem_pbar_wrap -->
</div><!-- #progressbar -->
<?php endif; ?>
<!-- <?php echo get_option('tpc_memory_usage_graph'); ?> -->
<p id="tpc_memory_usage_more_button" style="font-size: 9px;"><a href="javascript:void(0)">More...</a></p>
<p id="tpc_memory_usage_hide_button" class="hidden" style="font-size: 9px;"><a href="javascript:void(0)">Hide...</a></p>
<div id="tpc_memory_usage_more" class="hidden">
	<ul>
		<li><strong><?php esc_html_e('PHP Version'); ?></strong>: <span><?php echo PHP_VERSION; ?></span></li>
		<li><strong><?php esc_html_e('Server Software'); ?></strong>: <span><?php echo $_SERVER['SERVER_SOFTWARE']; ?></span></li>
		<li><strong><?php esc_html_e('User Agent'); ?></strong>: <span><?php echo $_SERVER['HTTP_USER_AGENT']; ?></span></li>
	</ul>
</div>

<?php
}

/**
 * Display TPC! Memory Usage JavaScript in 'admin_footer'.
 * 
 * @return void
 */
function _tpc_memory_usage_dashboard_js() {
	wp_print_scripts( array('tpc-memory-usage') );
}

/**
 * Displays memory usage comments in HTML.
 * 
 * @uses $tpcmem TPC! Memory Usage object
 */
function _tpc_memory_usage() {
	global $tpcmem;
?>

<!--
TPC! Memory Usage (http://webjawns.com)
<?php echo 'Memory Usage: ' . esc_html( $tpcmem->get_usage(true) ) . "\r"; ?>
<?php echo 'Memory Peak Usage: ' . esc_html( $tpcmem->get_peak_usage(true) ) . "\r"; ?>
<?php echo 'WP Memory Limit: ' . esc_html( $tpcmem->get_limit() ) . "\r"; ?>
<?php echo 'PHP Memory Limit: ' . esc_html( $tpcmem->get_limit(false) ) . "\r"; ?>
-->

<?php
}

/**
 * Converts bytes to megabytes.
 * 
 * @param int The number of bytes.
 * @return int The number of megabytes.
 */
function tpc_bytes_to_mb($value) {
	if ( $value == 0 )
		return 0;
	
	$value = round( $value / 1024 / 1024, 2 );
	
	return $value;
}

?>